#!/usr/bin/python3

BASE = '/usr/share/cruft/rules'

import glob
import os
import subprocess

STABLE = os.path.basename(os.readlink('archive/stable'))

for rule in glob.glob('rules/*'):
    package = os.path.basename(rule)
    dest = os.path.join(BASE, package)
    archive = 'archive/%s/%s' % (STABLE, package)
    if os.path.isfile(dest) and not os.path.exists(archive):
        print('%s has been dh-cruft enabled' % package)
        subprocess.call(['diff', '-u', rule, dest])
        subprocess.call(['git', 'mv', rule, archive])



#!/usr/bin/python3

from datetime import datetime
import os

import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import matplotlib.ticker as mtick

from git import Repo

def graph(package: str, rules: str) -> tuple[list[datetime],list[int]]:
    repo = Repo.init('/home/tchet/deb/%s' % package)
    count = dict()

    for c in repo.iter_commits(all=True, paths=[rules]):
        blob = c.tree[rules]
        count[c.authored_date] = len(blob.list_traverse())

    stamps = []
    values = []
    for k, v in sorted(count.items()):
        stamps.append(datetime.fromtimestamp(k))
        values.append(v)
    return stamps, values

st_old, c_old = graph('cruft', 'filters-unex')
st_new, c_new = graph('cruft-ng', 'rules')

# add data from https://snapshot.debian.org/binary/cruft/
#
# for d in *.deb
# do
#   echo -n $(echo $d | cut -d_ -f 2) " "
#   dpkg --fsys-tarfile $d | tar -tl --wildcards '*usr/lib/cruft/filters*' | grep -v 'usr/lib/cruft/filters/$'| wc -l
# done
SEED: dict[datetime, int] = {
    datetime(1998, 4, 10): 81, # https://lists.debian.org/debian-policy/1998/04/msg00089.html
    datetime(1998, 11, 9): 112, # 0.9.5
    datetime(1999, 12, 19): 112, # 0.9.6-0.1
    datetime(2002, 11, 22): 112, # 0.9.6-0.4
    datetime(2005, 7, 16): 129, # 0.9.6-0.5
    datetime(2005, 7, 20): 129, # 0.9.6-0.6
    datetime(2005, 7, 25): 129, # 0.9.6-0.7
    datetime(2005, 8, 29): 172, # 0.9.6-0.8
    datetime(2005, 10, 25): 156, # 0.9.6-0.9
    datetime(2005, 10, 28): 175, # 0.9.6-0.10
    datetime(2005, 11, 2): 185, # 0.9.6-0.11
    datetime(2006, 2, 6): 208, # 0.9.6-0.13
    datetime(2006, 6, 8): 208, # 0.9.6-0.14
    datetime(2006, 6, 15): 208, # 0.9.6-0.15
    datetime(2007, 1, 18): 210, # 0.9.6-0.16
    datetime(2007, 3, 3): 210, # 0.9.6-0.17
}

# split cruft <-> cruft-common
CUTOFF = datetime(2015, 6, 29)
st_cruft = list()
c_cruft = list()
for k, v in sorted(SEED.items()):
    st_cruft.append(k)
    c_cruft.append(v)

st_common = list()
c_common = list()
for st, c in zip(st_old, c_old):
    if st < CUTOFF:
       st_cruft.append(st)
       c_cruft.append(c)
    else:
       st_common.append(st)
       c_common.append(c)


fig, ax = plt.subplots()
ax.grid(True)
ax.plot(st_cruft, c_cruft, label='cruft')
ax.plot(st_common, c_common, label='cruft-common')
ax.plot(st_new, c_new, label='cruft-ng')

#ax.set_ylim(0, 615)

fig.autofmt_xdate()

plt.legend()
plt.show()



#!/usr/bin/python3

# rules that exists but do not match anything

import glob
import os

def is_installed(package: str) -> bool:
    for suffix in ('', ':amd64', ':i386'):
        if os.path.exists('/var/lib/dpkg/info/%s%s.list' % (package, suffix)):
            return True
    else:
        return False

for rule in sorted(glob.glob('rules/*')):
    package = os.path.basename(rule)
    if is_installed(package):
        with open(rule) as fd:
            for path in fd:
                path = path.strip('\n')
                if path.startswith('#'):
                    pass
                elif os.path.exists(path):
                    pass
                elif glob.glob(path):
                    pass
                else:
                    print(package, path)



#!/usr/bin/python3
# encoding=utf-8
#
# Copyright © 2015 Alexandre Detiste <alexandre@detiste.be>
# SPDX-License-Identifier: GPL-2.0-or-later
#
# https://launchpad.net/~alexandre-detiste/+archive/ubuntu/ppa

import os
import subprocess
import sys
import time

import distro_info

CRUFT = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
BASE = os.path.dirname(CRUFT)

info = distro_info.UbuntuDistroInfo()
stable = str(info.stable())
lts = str(info.lts())
old_lts = str(info.supported()[-3])

releases = sorted(set([old_lts, lts, stable]))
if len(sys.argv) > 1:
    release = sys.argv[1]
    assert release in releases
    releases = [release]

print('RELEASES:', releases)

subprocess.check_call(['git', 'checkout', 'debian/changelog'],
                       cwd = CRUFT)
subprocess.check_call(['git', 'checkout', 'debian/control'],
                       cwd = CRUFT)

from debian.changelog import Changelog
with open('debian/changelog', encoding='utf-8') as log:
    cl = Changelog(log, strict=False)

assert cl.distributions in ('unstable','UNRELEASED'), cl.distributions

if cl.distributions == 'unstable':
   build_type = 'Backport to PPA'
   build_number = 'release+'
else:
   build_type = 'Git snapshot'
   build_number = time.strftime('git%Y%m%d+')

with open('debian/control', 'r') as compat:
    for line in compat:
        if 'debhelper-compat' in line:
            current_debhelper =  int(line.split('(')[1].strip(' =),\n'))
            break

for release in sorted(releases):

    supported_debhelper = {
            'focal': 12,
            }.get(release, current_debhelper)
    BACKPORT = supported_debhelper < current_debhelper

    if BACKPORT:
        build_dep = 'debhelper-compat ( = %d)' %  supported_debhelper
        subprocess.check_call(['sed', '-i',
                               r's/\ *debhelper-compat.*/ ' + build_dep + ',/',
                               'debian/control'],
                              cwd = CRUFT)

    snapshot = str(cl.version).split('~')[0] + '~' + build_number + release
    subprocess.check_call(['dch', '-b',
                           '-D', release,
                           '-v', snapshot,
                           build_type],
                          cwd = CRUFT)

    subprocess.check_call(['debuild', '--no-lintian', '-S', '-i'], cwd = CRUFT)
    subprocess.check_call(['dput', 'my-ppa',
                           'cruft-ng_%s_source.changes' % snapshot],
                           cwd = BASE)

    subprocess.check_call(['git', 'checkout', 'debian/changelog'],
                          cwd = CRUFT)
    if BACKPORT:
        subprocess.check_call(['git', 'checkout', 'debian/control'],
                               cwd = CRUFT)
        subprocess.check_call(['git', 'checkout', 'debian/dh-cruft.manpages'],
                               cwd = CRUFT)

    for file in ('.tar.xz',
                 '.dsc',
                 '_source.build',
                 '_source.changes',
                 '_source.my-ppa.upload'):
        subprocess.check_call(['rm', '-v',
                               'cruft-ng_%s%s' % (snapshot, file)],
                              cwd = BASE)



#!/usr/bin/python3

import os

import requests
from bs4 import BeautifulSoup

URL = 'https://mentors.debian.net/'

r = requests.get(URL)
soup = BeautifulSoup(r.content, "html.parser")

for a in soup.find_all('a'):
    href = a['href']
    if href.startswith('/package/'):
        package = href.split('/')[2]
        if os.path.exists('rules/%s' % package):
            print(package)



#!/usr/bin/python3

import glob
import re
import sys

import debianbts as bts

# Shell: /bin/sh linked to /bin/dash
FALSE_POSITIVES = set(['/bin/sh', '/bin/dash',
                   '/usr/bin/sh', '/usr/bin/dash',
                   '/usr/sbin/piuparts',
                   '/usr/share/fonts',
                  ])

for dpkg_list in glob.glob('/var/lib/dpkg/info/*.list') + glob.glob('/var/lib/dpkg/info/*.conffiles'):
    with open(dpkg_list, 'r') as files:
        for file in files:
            FALSE_POSITIVES.add(file.rstrip('\n'))

# exclude "'" because it's added by dpkg in error messages
re_cruft = re.compile(r'/(?:bin|usr|etc|var|\.cache)/[A-Za-z0-9_\/\.\-]*')

# https://udd.debian.org/cgi-bin/bts-usertags.cgi?user=cruft-ng@packages.debian.org&tag=cruft
if len(sys.argv) > 1:
    bugs = [int(bug) for bug in sys.argv[1:]]
else:
    bugs = bts.get_usertag('cruft-ng@packages.debian.org', tags=['cruft'])['cruft']

cruft = dict()

todo: set[int] = set(bugs)
done: set[int] = set()

while todo:
    todo -= done
    for bug in bts.get_status(list(todo)):
        todo.update(set(bug.mergedwith))
        done.add(bug.bug_num)
        for match in re.findall(re_cruft, bug.subject):
            match = match.rstrip('./')
            if match in cruft:
                continue
            cruft[match] = (bug.bug_num, bug.source, bug.subject)
        for mail in bts.get_bug_log(bug.bug_num):
            for line in mail['body'].splitlines():
                if '---- missing: dpkg ----' in line:
                    # bug report contains cruft report, ignore from here
                    break
                for match in re.findall(re_cruft, line):
                    if match in FALSE_POSITIVES:
                        continue
                    if match.startswith('/var/lib/dpkg/'):
                        continue
                    if '  interest-noawait ' in line:
                        continue
                    match = match.rstrip('./')
                    if match in cruft:
                        # keep original bug report
                        continue
                    cruft[match] = (bug.bug_num, bug.source, line)

for path,data in sorted(cruft.items()):
    bug_nr, package, raw = data
    #print(raw)
    print('%s %s %s' % (path, bug_nr, package))



#!/usr/bin/python3

import os
import subprocess

STABLE = os.path.basename(os.readlink('archive/stable'))

for package in subprocess.check_output(['apt-file', 'search', '--package-only', '/usr/share/cruft/rules/'],
                                       text=True).splitlines():
    rule = os.path.join('rules', package)
    archive = 'archive/%s/%s' % (STABLE, package)
    if os.path.isfile(rule) and not os.path.isfile(archive):
        print('%s has been dh-cruft enabled' % package)
        subprocess.call(['git', 'mv', rule, archive])



#!/usr/bin/python3

import os
import subprocess
import sys

try:
   PATH = sys.argv[1]
   assert PATH in ('cache', 'games', 'lib', 'log', 'spool')
except:
   exit('tools/codesearch.py cache|games|lib|log|spool')

import apt
CACHE = apt.Cache()

def bin2src(binaries: list[str]) -> list[str]:
    '''compare source packages with source packages'''
    sources = []
    for binary in binaries:
        try:
            package = CACHE[binary]
            source = package.candidate.source_name
        except KeyError:
            #print('!!! %s' % binary) # does not exists in Debian
            source = binary
        sources.append(source)
    return sources

done = subprocess.check_output(['apt-file', 'search', '--package-only', '/usr/share/cruft/rules/'], text=True).splitlines()
pending = os.listdir('rules/') + os.listdir('explain/')
pending = [p for p in sorted(pending) if p.lower() == p]  # ignore DH_CRUFT & DEBOOTSTRAP
todo = subprocess.check_output(['codesearch', 'path:debian/.*postrm /var/%s' % PATH, '--only', '--max-result', '500'], text=True).splitlines()

done = bin2src(done)
pending = bin2src(pending)
to_check = set(todo) - set(done) - set(pending)
if to_check:
    print('\n'.join(sorted(to_check)))



#!/usr/bin/python3

# /usr/share/lintian/data/fields/obsolete-packages

import glob
import os
import subprocess

import apt
import distro_info

STABLE = str(distro_info.DebianDistroInfo().stable())

def check_for_new_release() -> None:
    '''usefull once every two year'''
    old_stable = os.path.basename(os.readlink('archive/stable'))
    if old_stable != STABLE:
        print('Stable distribution has changed: %s -> %s' % (old_stable, STABLE))
        os.unlink('archive/stable')
        os.chdir('archive')
        if not os.path.isdir(STABLE):
            os.mkdir(STABLE)
            gitkeep = os.path.join(STABLE, '.gitkeep')
            open(gitkeep, 'w').close()
            subprocess.call(['git', 'add', gitkeep])
        os.symlink(STABLE, 'stable')
        os.chdir('..')

    if old_stable != STABLE or not os.path.isfile('tools/Packages_amd64'):
        subprocess.check_call(['ben', 'download',
                               '--archs', 'amd64',
                               '--suite', 'stable'],
                              cwd='tools')
        os.unlink('tools/Sources')

check_for_new_release()

def get_obsolete() -> dict[str, str]:
    obsolete = dict()
    with open('/usr/share/lintian/data/fields/obsolete-packages', 'r') as fd:
         for line in fd:
             line = line.strip()
             if not line:
                 continue
             if line.startswith('#'):
                 continue
             old = line.split()[0]
             try:
                 new_ = line.split(maxsplit=1)[1]
             except IndexError:
                 new_ = ''
             obsolete[old] = new_
    return obsolete

obsolete = get_obsolete()

def get_testing() -> set[str]:
    testing = set()
    c = apt.Cache()
    for p in c.keys():
        o = c[p]
        if o.candidate:
            d = o.candidate.record['Description'].lower()
            if 'transitional' in d or 'dummy' in d:
                pass
            else:
               testing.add(o.shortname)
    return testing

testing = get_testing()

testing.add('usr-is-merged')

# arm64 only
testing.add('widevine-installer')

# Add RaspBian
testing.add('raspberrypi-bootloader')
testing.add('libraspberrypi0')

# Add BeagleBone
testing.add('bb-customizations')
testing.add('bb-node-red-installer')
testing.add('c9-core-installer')
testing.add('doc-beaglebone-getting-started')
testing.add('librobotcontrol')
testing.add('ti-pru-cgt-installer')

def get_stable() -> set[str]:
    stable = set()
    with open('tools/Packages_amd64', 'r') as fd:
        for line in fd:
            if line.startswith('Package:'):
                stable.add(line.split(':', 1)[1].strip())
    return stable

stable = get_stable()


filters = set([os.path.basename(f) for f in glob.glob('rules/*') if f == f.lower()])
explain = set([os.path.basename(f) for f in glob.glob('explain/*') if f == f.lower()])

def process(
    category: str,
    packages: set[str],
    testing: set[str],
    stable: set[str],
    obsolete: dict[str, str],
    archive: str
) -> None:
    unknown = packages - testing
    deprecated = unknown & stable
    unknown = unknown - stable
    print('[%s]:' % category)
    print('moved to %s:' % STABLE, sorted(deprecated))
    for item in deprecated:
        destdir = os.path.join('archive', archive)
        if not os.path.isdir(destdir):
            os.makedirs(destdir)
        subprocess.call(['git', 'mv', os.path.join(category, item), destdir])
        gitkeep = os.path.join(destdir, '.gitkeep')
        if os.path.isfile(gitkeep):
            subprocess.call(['git', 'rm', gitkeep])
    print('unknown:', sorted(unknown))
    o = packages & obsolete.keys()
    print('obsolete:', sorted(o))

process('rules', filters, testing, stable, obsolete, STABLE)
print()
process('explain', explain, testing, stable, obsolete, 'explain')
